/* Copyright (c) 2004 David Hammerton - All rights reserved
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */


#include <gtk/gtk.h>
#include <glade/glade.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

#include "lists.h"
#include "tunesbrowser.h"

static int update_ui[2] = {-1, -1};

static GtkLabel *playstatus;
static GtkLabel *play_str;

static GtkLabel *playpause_label;
static GtkImage *playpause_image;

enum songstatus currentStatus = STOPPED;

enum UPDATE_TYPE
{
    UPDATE_SONGSTATUS,
    UPDATE_SONGPOS,
    UPDATE_HOSTS
};

/****************** insert message ************/
void set_songstatus(enum songstatus status, int songid)
{
    enum UPDATE_TYPE type = UPDATE_SONGSTATUS;
    write(update_ui[1], &type, sizeof(enum UPDATE_TYPE));
    write(update_ui[1], &status, sizeof(int));
    if (status == PLAYING || status == PAUSED)
        write(update_ui[1], &songid, sizeof(int));
}

void set_songpos(int seconds)
{
    enum UPDATE_TYPE type = UPDATE_SONGPOS;
    write(update_ui[1], &type, sizeof(enum UPDATE_TYPE));
    write(update_ui[1], &seconds, sizeof(int));
}

void set_hostsupdated()
{
    enum UPDATE_TYPE type = UPDATE_HOSTS;
    write(update_ui[1], &type, sizeof(enum UPDATE_TYPE));
}

/******** callback handlers *******/
static void cb_update_sources()
{
    finalize_daap_sources();
    init_daap_sources();
}

static void cb_update_songstatus()
{
    enum songstatus status;
    int ret = read(update_ui[0], &status, sizeof(int));
    if (ret == -1)
        return;

    currentStatus = status;

    switch(status)
    {
        case STOPPED:
            gtk_label_set_text(play_str, "Not Playing");
            gtk_label_set_text(playstatus, "-");

            /* fall through */
        case PAUSED:
            gtk_label_set_text(playpause_label, "Play");
            gtk_image_set_from_stock(playpause_image, "gtk-yes", 4);

            if (status == STOPPED)
                break;

            /* fall through */
        case PLAYING:
            {
                int songid = 0;
                char *str;
                ret = read(update_ui[0], &songid, sizeof(int));
                str = get_song_display_str_withalloc(songid);
                gtk_label_set_text(play_str, str);
                free(str);
            }
            if (status == PAUSED)
                break;
            gtk_label_set_text(playpause_label, "Pause");
            gtk_image_set_from_stock(playpause_image, "gtk-no", 4);

            break;
    }
}

static void cb_update_songpos()
{
    int seconds;
    int ret = read(update_ui[0], &seconds, sizeof(int));
    if (ret == -1)
        return;

    if (currentStatus == STOPPED)
    {
        return;
    }

    if (seconds >= 0)
    {
        int minutes = seconds / 60;
        seconds -= minutes * 60;
        char bufstr[11] = {0};

        snprintf(bufstr, 10, "%i:%02i",
                 minutes, seconds);

        gtk_label_set_text(playstatus, bufstr);
    }
    else
    {
        gtk_label_set_text(playstatus, "-");
    }
}

/********* callback pump ***********/
static gboolean ui_update_cb(GIOChannel *source,
                             GIOCondition condition,
                             gpointer data)
{
    enum UPDATE_TYPE type;
    int ret;

    if (condition != G_IO_IN)
        return TRUE;

    while ((ret = read(update_ui[0], &type, sizeof(enum UPDATE_TYPE)))
            != -1)
    {
        fcntl(update_ui[0], F_SETFL, 0);
        switch (type)
        {
            case UPDATE_SONGSTATUS:
                cb_update_songstatus();
                break;
            case UPDATE_SONGPOS:
                cb_update_songpos();
                break;
            case UPDATE_HOSTS:
                cb_update_sources();
                break;
        }
        fcntl(update_ui[0], F_SETFL, O_NONBLOCK);
    }
    if (errno != EAGAIN)
        fprintf(stderr, "expected EAGAIN error\n");

    return TRUE;
}

/********** standard callbacks *******/
void on_playpause_clicked(GtkButton *playpause, gpointer user_data)
{
    if (currentStatus == PLAYING)
    {
        pause_daap_song();
    }
    else if (currentStatus == STOPPED)
    {
        play_daap_song(songlist_get_selected_id());
    }
    else if (currentStatus == PAUSED)
    {
        resume_daap_song();
    }
}

/************** init **************/

void misc_ui_init(GladeXML *xml)
{
    GIOChannel *ioc;
    GtkButton *playpause;

    playstatus = GTK_LABEL(glade_xml_get_widget(xml, "playstatus"));
    play_str = GTK_LABEL(glade_xml_get_widget(xml, "play_str"));

    playpause_label = GTK_LABEL(glade_xml_get_widget(xml, "playpause_label"));
    playpause_image = GTK_IMAGE(glade_xml_get_widget(xml, "playpause_image"));

    playpause = GTK_BUTTON(glade_xml_get_widget(xml, "playpause"));
    g_signal_connect(G_OBJECT(playpause), "clicked",
                     G_CALLBACK(on_playpause_clicked),
                     NULL);

    pipe(update_ui);
    fcntl(update_ui[0], F_SETFL, O_NONBLOCK);
    ioc = g_io_channel_unix_new(update_ui[0]);
    g_io_add_watch(ioc, G_IO_IN,
                   ui_update_cb, NULL);
}

